@@ -69,16 +69,85 @@
 #define OFX_SECNAME_LENGTH             32 + 1
 #define OFX_TICKER_LENGTH              32 + 1
 
-CFCT void (*OfxCallbackFunc) ();
+/*
+#define OFX_STATEMENT_CB               0;
+#define OFX_ACCOUNT_CB                 1;
+#define OFX_TRACSACTION_CB             2;
+#define OFX_SECURITY_CB                3;
+#define OFX_STATUS_CB                  4;
+*/
 
+typedef void * LibofxContextPtr;
+/**
+ * \brief Initialise the library and return a new context. 
+ *
+ @return the new context, to be used by the other functions.
+*/
+CFCT LibofxContextPtr libofx_get_new_context();
+/**
+ * \brief Free all ressources used by this context. 
+ *
+ @return 0 if successfull.
+*/
+CFCT int libofx_free_context( LibofxContextPtr );
+
+/** List of possible file formats */
+enum LibofxFileFormat{ AUTODETECT, /**< Not really a file format, used to tell the library to try to autodetect the format*/
+		     OFX, /**< Open Financial eXchange (OFX/QFX) file */
+		     OFC, /**< Microsoft Open Financial Connectivity (OFC)*/
+		     QIF, /**< Intuit Quicken Interchange Format (QIF) */
+		     UNKNOWN, /**< Unknown file format */
+		     LAST /**< Not a file format, meant as a loop breaking condition */
+};
+
+struct LibofxFileFormatInfo{
+enum LibofxFileFormat format;/**< The file format enum */
+const char * format_name;  /**< Text version of the enum */
+const char * description; /**< Description of the file format */
+};
+
+const struct LibofxFileFormatInfo LibofxImportFormatList[] = 
+{
+{AUTODETECT, "AUTODETECT", "AUTODETECT (File format will be automatically detected later)"},
+{OFX, "OFX", "OFX (Open Financial eXchange (OFX or QFX))"},
+{OFC, "OFC", "OFC (Microsoft Open Financial Connectivity)"},
+{QIF, "QIF", "QIF (Intuit Quicken Interchange Format) NOT IMPLEMENTED"},
+{LAST, "LAST", "Not a file format, meant as a loop breaking condition"}
+};
+
+const struct LibofxFileFormatInfo LibofxExportFormatList[] = 
+{
+{QIF, "QIF", "QIF (Intuit Quicken Interchange Format) NOT IMPLEMENTED"},
+{LAST, "LAST", "Not a file format, meant as a loop breaking condition"}
+};
 
 /**
- * \brief ofx_proc_file is the entry point of the library.  
+ * \brief libofx_get_file_type returns a proper enum from a file type string. 
+ *
+ @format_list The file format list in which the format string should be found, usually LibofxImportFormatList or LibofxExportFormatList
+ @file_type_string The string which contain the file format matching one of the format_name of the list.
+ @return the file format, or UNKNOWN if the format wasn't recognised.
+*/
+CFCT enum LibofxFileFormat libofx_get_file_format_from_str(const struct LibofxFileFormatInfo format_list[], const char * file_type_string);
+
+/**
+ * \brief get_file_format_description returns a string description of a LibofxFileType. 
+ *
+ @format_list The file format list in which the format should be looked up, usually LibofxImportFormatList or LibofxExportFormatList
+ @file_format The file format which should match one of the formats in the list.
+ @return null terminated string suitable for debugging output or user communication.
+*/
+CFCT const char * libofx_get_file_format_description(const struct LibofxFileFormatInfo format_list[], enum LibofxFileFormat file_format);
+
+/**
+ * \brief libofx_proc_file is the entry point of the library.
  *
  *  libofx_proc_file must be called by the client, with a list of 1 or more OFX
  files to be parsed in command line format.
 */
-CFCT int ofx_proc_file(int argc, char *argv[]);
+CFCT int libofx_proc_file(LibofxContextPtr libofx_context,
+                          const char * p_filename,
+                          enum LibofxFileFormat ftype);
 
 
 /**
@@ -135,9 +204,10 @@ struct OfxStatusData{
  *
  * An ofx_proc_status_cb event is sent everytime the server has generated a OFX
  STATUS element.  As such, it could be received at any time(but not during
- other events).  An OfxStatusData structure is passed to this even.
+ other events).  An OfxStatusData structure is passed to this event, as well as
+ a pointer to an arbitrary data structure.
 */
-CFCT int ofx_proc_status_cb(const struct OfxStatusData data);
+CFCT typedef int (*LibofxProcStatusCallback)(const struct OfxStatusData data, void * status_data);
 
 /**
  * \brief An abstraction of an account
@@ -191,9 +261,10 @@ struct OfxAccountData{
  received.  An OfxAccountData is passed to this event.
  *
  Note however that this OfxAccountData structure will also be available as
- part of OfxStatementData structure passed to  ofx_proc_statement event.
+ part of OfxStatementData structure passed to ofx_proc_statement event, 
+ as well as a pointer to an arbitrary data structure.
 */
-CFCT int ofx_proc_account_cb(const struct OfxAccountData data);
+CFCT typedef int (*LibofxProcAccountCallback)(const struct OfxAccountData data, void * account_data);
 
 /**
  * \brief An abstraction of a security, such as a stock, mutual fund, etc.
@@ -246,31 +317,12 @@ struct OfxSecurityData{
  ofx_proc_transaction. It is meant to be used to allow the client to 
  create a new commodity or security (such as a new stock type).  Please note however
  that this information is usually also available as part of each OfxtransactionData.
- An OfxSecurityData structure is passed to this event.
+ An OfxSecurityData structure is passed to this event, as well as
+ a pointer to an arbitrary data structure.
 */
-CFCT int ofx_proc_security_cb(const struct OfxSecurityData data);
+CFCT typedef int (*LibofxProcSecurityCallback)(const struct OfxSecurityData data, void * security_data);
 
-
-/**
- * \brief An abstraction of a transaction in an account.
- *
- * The OfxTransactionData stucture contains all available information about
- an actual transaction in an account.
-*/
-struct OfxTransactionData{
-  
-  /** @name OFX mandatory elements
-   * The OFX spec defines the following elements as mandatory.  The associated
-   variables should all contain valid data but you should not trust the servers.
-   Check if the associated *_valid is true before using them. */  
-
-  char account_id[OFX_ACCOUNT_ID_LENGTH];/**< Use this for matching with
-					    the relevant account in your
-					    application */
-  struct OfxAccountData * account_ptr; /**< Pointer to the full account structure,
-					  see OfxAccountData */
-  int account_id_valid;
-  enum TransactionType{
+typedef enum {
     OFX_CREDIT,     /**< Generic credit */
     OFX_DEBIT,      /**< Generic debit */
     OFX_INT,        /**< Interest earned or paid (Note: Depends on signage of amount) */
@@ -288,13 +340,9 @@ struct OfxTransactionData{
     OFX_DIRECTDEBIT,/**< Merchant initiated debit */
     OFX_REPEATPMT,  /**< Repeating payment/standing order */
     OFX_OTHER       /**< Somer other type of transaction */
-  } transactiontype;
-  int transactiontype_valid;
- 
-  /**< Investment transaction type.  You should read this if 
-     transactiontype == OFX_OTHER.  See OFX spec 1.6 p.442 to 445 
-     for details*/
-  enum InvTransactionType{
+  } TransactionType;
+
+typedef  enum{
     OFX_BUYDEBT,        /**< Buy debt security */
     OFX_BUYMF,          /**< Buy mutual fund */
     OFX_BUYOPT,         /**< Buy option */
@@ -315,9 +363,42 @@ struct OfxTransactionData{
     OFX_SELLSTOCK,      /**< Sell stock */
     OFX_SPLIT,          /**< Stock or mutial fund split */
     OFX_TRANSFER        /**< Transfer holdings in and out of the investment account */
-  }  invtransactiontype;
+  }  InvTransactionType;
+typedef enum {
+    DELETE, /**< The transaction with a fi_id matching fi_id_corrected should
+	       be deleted */
+    REPLACE /**< The transaction with a fi_id matching fi_id_corrected should 
+	       be replaced with this one */
+  } FiIdCorrectionAction;
 
- int  invtransactiontype_valid;
+/**
+ * \brief An abstraction of a transaction in an account.
+ *
+ * The OfxTransactionData stucture contains all available information about
+ an actual transaction in an account.
+*/
+struct OfxTransactionData{
+  
+  /** @name OFX mandatory elements
+   * The OFX spec defines the following elements as mandatory.  The associated
+   variables should all contain valid data but you should not trust the servers.
+   Check if the associated *_valid is true before using them. */  
+
+  char account_id[OFX_ACCOUNT_ID_LENGTH];/**< Use this for matching with
+					    the relevant account in your
+					    application */
+  struct OfxAccountData * account_ptr; /**< Pointer to the full account structure,
+					  see OfxAccountData */
+  int account_id_valid;
+
+  TransactionType transactiontype;
+  int transactiontype_valid;
+ 
+  /**< Investment transaction type.  You should read this if 
+     transactiontype == OFX_OTHER.  See OFX spec 1.6 p.442 to 445 
+     for details*/
+  InvTransactionType invtransactiontype;
+  int  invtransactiontype_valid;
 
   double units;     /**< Variation of the number of units of the commodity
 		     Suppose units is -10, ave unitprice is 1.  If the 
@@ -379,12 +460,7 @@ struct OfxTransactionData{
   int fi_id_corrected_valid;
   /** The OfxTransactionData::FiIdCorrectionAction enum contains the action
       to be taken */
-  enum FiIdCorrectionAction{
-    DELETE, /**< The transaction with a fi_id matching fi_id_corrected should
-	       be deleted */
-    REPLACE /**< The transaction with a fi_id matching fi_id_corrected should 
-	       be replaced with this one */
-  } fi_id_correction_action;
+  FiIdCorrectionAction fi_id_correction_action;
   int fi_id_correction_action_valid;
   
   /** Used for user initiated transaction such as payment or funds transfer.
@@ -419,9 +495,10 @@ char check_number[OFX_CHECK_NUMBER_LENGTH];
  *
  * An ofx_proc_transaction_cb event is generated for every transaction in the 
  ofx response, after ofx_proc_statement (and possibly ofx_proc_security is 
- generated. An OfxTransactionData structure is passed to this event.
+ generated. An OfxTransactionData structure is passed to this event, as well as
+ a pointer to an arbitrary data structure.
 */
-CFCT int ofx_proc_transaction_cb(const struct OfxTransactionData data);
+CFCT typedef int (*LibofxProcTransactionCallback)(const struct OfxTransactionData data, void * transaction_data);
 
 /** 
  * \brief An abstraction of an account statement. 
@@ -487,9 +564,10 @@ struct OfxStatementData{
  * \brief The callback function for the OfxStatementData stucture. 
  *
  * The ofx_proc_statement_cb event is sent after all ofx_proc_transaction 
- events have been sent. An OfxStatementData is passed to this event.
+ events have been sent. An OfxStatementData is passed to this event, as well as
+ a pointer to an arbitrary data structure.
 */
-CFCT int ofx_proc_statement_cb(const struct OfxStatementData data);
+CFCT typedef int (*LibofxProcStatementCallback)(const struct OfxStatementData data, void * statement_data);
 
 /** 
     \brief NOT YET SUPPORTED
@@ -500,5 +578,62 @@ struct OfxCurrency{
   int must_convert;   /**< true or false */
 };
 
+
+/**
+ * Set the status callback in the given context.
+ * @param ctx context
+ * @param cb callback function
+ * @param user_data user data to be passed to the callback
+ */
+CFCT void ofx_set_status_cb(LibofxContextPtr ctx,
+                            LibofxProcStatusCallback cb,
+                            void *user_data);
+/**
+ * Set the account callback in the given context.
+ * @param ctx context
+ * @param cb callback function
+ * @param user_data user data to be passed to the callback
+ */
+CFCT void ofx_set_account_cb(LibofxContextPtr ctx,
+                             LibofxProcAccountCallback cb,
+                             void *user_data);
+/**
+ * Set the security callback in the given context.
+ * @param ctx context
+ * @param cb callback function
+ * @param user_data user data to be passed to the callback
+ */
+CFCT void ofx_set_security_cb(LibofxContextPtr ctx,
+                              LibofxProcSecurityCallback cb,
+                              void *user_data);
+/**
+ * Set the transaction callback in the given context.
+ * @param ctx context
+ * @param cb callback function
+ * @param user_data user data to be passed to the callback
+ */
+CFCT void ofx_set_transaction_cb(LibofxContextPtr ctx,
+                                 LibofxProcTransactionCallback cb,
+                                 void *user_data);
+
+/**
+ * Set the statement callback in the given context.
+ * @param ctx context
+ * @param cb callback function
+ * @param user_data user data to be passed to the callback
+ */
+CFCT void ofx_set_statement_cb(LibofxContextPtr ctx,
+                               LibofxProcStatementCallback cb,
+                               void *user_data);
+
+
+/**
+ * Parses the content of the given buffer.
+ */
+CFCT int libofx_proc_buffer(LibofxContextPtr ctx,
+                            const char *s, unsigned int size);
+
+
+
 #endif
 
