@@ -1,7 +1,7 @@
 /***************************************************************************
                           ofxdump.cpp
                              -------------------
-    copyright            : (C) 2002 by Benoit Grégoire
+    copyright            : (C) 2002 by Benoit Grgoire
     email                : bock@step.polymtl.ca
 ***************************************************************************/
 /**@file
@@ -34,133 +34,12 @@
 #include <stdio.h>		/* for printf() */
 #include <config.h>		/* Include config constants, e.g., VERSION TF */
 
+#include "cmdline.h" /* Gengetopt generated parser */
 
 using namespace std;
 
 
-int main (int argc, char *argv[])
-{
-  /** Tell ofxdump what you want it to send to stderr.  See messages.cpp for more details */
-  extern int ofx_PARSER_msg;
-  extern int ofx_DEBUG_msg;
-  extern int ofx_WARNING_msg;
-  extern int ofx_ERROR_msg;
-  extern int ofx_INFO_msg;
-  extern int ofx_STATUS_msg;
-
-  ofx_PARSER_msg = false;
-  ofx_DEBUG_msg = false;
-  ofx_WARNING_msg = true;
-  ofx_ERROR_msg = true;
-  ofx_INFO_msg = true;
-  ofx_STATUS_msg = true;
-
-  int special_options (char *argv[]);
-  int count, ret = 0;;
- 
-  //printf ("This program was called with \"%s\".\n",argv[0]);
-  if (argc > 1) 
-    { 
-      for (count = 1; count < argc; count++) 
-	{
-	  //printf("argv[%d] = %s\n", count, argv[count]); 
-	}
-      if  (0 == special_options(argv)) ;
-      else  ofx_proc_file(argc, argv);    /* Special option not found */
-    } 
-  else
-    {
-      char *spoptar[] =           {"ofxdump","--help"};
-      special_options(spoptar);   /* No arguments             */
-    }
-  return 0; 
-}
-
-
-int special_options (char *argv[])
-{
-  int count;
-  
-  /* Define case numbers */
-  const int spoptver = 0;
-  const int spopthelp = 1; 
-  
-  /* Define option and case arrays */
-  const char *spoptar[] =           {"--version", "-V", "--help"};
-  const int spoptdim = sizeof(spoptar)/sizeof(spoptar[0]);
-  const int spoptcasear[spoptdim] = {spoptver, spoptver, spopthelp};
-  
-  /* Define help array  */
-  const char *helpar[] =
-    {
-      "ofxdump command synopsis:\n",
-      "ofxdump special_option\n",
-      "ofxdump ofx_file\n",
-      "\n",
-      "Special options are:\n",
-      "--version, V     libofx version\n",
-      "--help           help text\n",
-      "\n",
-      "If the first argument is a special option, it is processed and any remaining \
-arguments are ignored; otherwise, control goes to ofx_proc_file.\n\n"
-    };
-  const int helpardim = sizeof(helpar)/sizeof(helpar[1]);
-
-  /* Scan for special option */  
-  for (count = 0; (count < spoptdim) && (strcmp(argv[1], spoptar[count]) != 0) ; count++);
-
-  if (count < spoptdim)               /* If found */
-    {
-      switch (spoptcasear[count])
-	{ 
-	case spoptver:  printf ("libofx version:  %s \n", VERSION ); return 0;
-	case spopthelp: 
-	  {
-	    for (int i=0; i < helpardim; i++)  printf (helpar[i]); 
-	    return 0;
-	  }
-	default: 
-	  {
-	    printf("Impossible special option case\n");
-	    return -1;              /* Program error */
-	  }
-	}
-    }
-  return 1;    
-}
-
-
-
-
-int ofx_proc_status_cb(struct OfxStatusData data)
-{
-  cout<<"ofx_proc_status():\n";
-  if(data.ofx_element_name_valid==true){
-    cout<<"    Ofx entity this status is relevent to: "<< data.ofx_element_name<<" \n";
-  }
-  if(data.severity_valid==true){
-    cout<<"    Severity: ";
-    switch(data.severity){
-    case OfxStatusData::INFO : cout<<"INFO\n";
-      break;
-    case OfxStatusData::WARN : cout<<"WARN\n";
-      break;
-    case OfxStatusData::ERROR : cout<<"ERROR\n";
-      break;
-    default: cout<<"WRITEME: Unknown status severity!\n";
-    }
-  }
-  if(data.code_valid==true){
-    cout<<"    Code: "<<data.code<<", name: "<<data.name<<"\n    Description: "<<data.description<<"\n";
-  }
-  if(data.server_message_valid==true){
-    cout<<"    Server Message: "<<data.server_message<<"\n";
-  }
-  cout<<"\n";
-  return 0;
-}
-
-int ofx_proc_security_cb(struct OfxSecurityData data)
+int ofx_proc_security_cb(struct OfxSecurityData data, void * security_data)
 {
   char dest_string[255];
   cout<<"ofx_proc_security():\n";
@@ -193,7 +72,7 @@ int ofx_proc_security_cb(struct OfxSecurityData data)
   return 0;
 }
 
-int ofx_proc_transaction_cb(struct OfxTransactionData data)
+int ofx_proc_transaction_cb(struct OfxTransactionData data, void * transaction_data)
 {
   char dest_string[255];
   cout<<"ofx_proc_transaction():\n";
@@ -202,47 +81,48 @@ int ofx_proc_transaction_cb(struct OfxTransactionData data)
     cout<<"    Account ID : "<<data.account_id<<"\n";
   }
   
-  if(data.transactiontype_valid==true){
-    switch(data.transactiontype){
-    case data.OFX_CREDIT: strncpy(dest_string, "CREDIT: Generic credit", sizeof(dest_string));
-      break;
-    case data.OFX_DEBIT: strncpy(dest_string, "DEBIT: Generic debit", sizeof(dest_string));
-      break;
-    case data.OFX_INT: strncpy(dest_string, "INT: Interest earned or paid (Note: Depends on signage of amount)", sizeof(dest_string));
-      break;
-    case data.OFX_DIV: strncpy(dest_string, "DIV: Dividend", sizeof(dest_string));
-      break;
-    case data.OFX_FEE: strncpy(dest_string, "FEE: FI fee", sizeof(dest_string));
-      break;
-    case data.OFX_SRVCHG: strncpy(dest_string, "SRVCHG: Service charge", sizeof(dest_string));
-      break;
-    case data.OFX_DEP: strncpy(dest_string, "DEP: Deposit", sizeof(dest_string));
-      break;
-    case data.OFX_ATM: strncpy(dest_string, "ATM: ATM debit or credit (Note: Depends on signage of amount)", sizeof(dest_string));
-      break;
-    case data.OFX_POS: strncpy(dest_string, "POS: Point of sale debit or credit (Note: Depends on signage of amount)", sizeof(dest_string));
-      break;
-    case data.OFX_XFER: strncpy(dest_string, "XFER: Transfer", sizeof(dest_string));
-      break;
-    case data.OFX_CHECK: strncpy(dest_string, "CHECK: Check", sizeof(dest_string));
-      break;
-    case data.OFX_PAYMENT: strncpy(dest_string, "PAYMENT: Electronic payment", sizeof(dest_string));
-      break;
-    case data.OFX_CASH: strncpy(dest_string, "CASH: Cash withdrawal", sizeof(dest_string));
-      break;
-    case data.OFX_DIRECTDEP: strncpy(dest_string, "DIRECTDEP: Direct deposit", sizeof(dest_string));
-      break;
-    case data.OFX_DIRECTDEBIT: strncpy(dest_string, "DIRECTDEBIT: Merchant initiated debit", sizeof(dest_string));
-      break;
-    case data.OFX_REPEATPMT: strncpy(dest_string, "REPEATPMT: Repeating payment/standing order", sizeof(dest_string));
-      break;
-    case data.OFX_OTHER: strncpy(dest_string, "OTHER: Other", sizeof(dest_string));
-      break;
-    default : strncpy(dest_string, "Unknown transaction type", sizeof(dest_string));
-      break;
+  if(data.transactiontype_valid==true)
+    {
+      if(data.transactiontype==OFX_CREDIT)
+	strncpy(dest_string, "CREDIT: Generic credit", sizeof(dest_string));
+      else if (data.transactiontype==OFX_DEBIT)
+	strncpy(dest_string, "DEBIT: Generic debit", sizeof(dest_string));
+      else if (data.transactiontype==OFX_INT)
+	strncpy(dest_string, "INT: Interest earned or paid (Note: Depends on signage of amount)", sizeof(dest_string));
+      else if (data.transactiontype==OFX_DIV)
+	strncpy(dest_string, "DIV: Dividend", sizeof(dest_string));
+      else if (data.transactiontype==OFX_FEE)
+	strncpy(dest_string, "FEE: FI fee", sizeof(dest_string));
+      else if (data.transactiontype==OFX_SRVCHG)
+	strncpy(dest_string, "SRVCHG: Service charge", sizeof(dest_string));
+      else if (data.transactiontype==OFX_DEP)
+	strncpy(dest_string, "DEP: Deposit", sizeof(dest_string));
+      else if (data.transactiontype==OFX_ATM)
+	strncpy(dest_string, "ATM: ATM debit or credit (Note: Depends on signage of amount)", sizeof(dest_string));
+      else if (data.transactiontype==OFX_POS)
+	strncpy(dest_string, "POS: Point of sale debit or credit (Note: Depends on signage of amount)", sizeof(dest_string));
+      else if (data.transactiontype==OFX_XFER)
+	strncpy(dest_string, "XFER: Transfer", sizeof(dest_string));
+      else if (data.transactiontype==OFX_CHECK)
+	strncpy(dest_string, "CHECK: Check", sizeof(dest_string));
+      else if (data.transactiontype==OFX_PAYMENT)
+	strncpy(dest_string, "PAYMENT: Electronic payment", sizeof(dest_string));
+      else if (data.transactiontype==OFX_CASH)
+	strncpy(dest_string, "CASH: Cash withdrawal", sizeof(dest_string));
+      else if (data.transactiontype==OFX_DIRECTDEP)
+	strncpy(dest_string, "DIRECTDEP: Direct deposit", sizeof(dest_string));
+      else if (data.transactiontype==OFX_DIRECTDEBIT)
+	strncpy(dest_string, "DIRECTDEBIT: Merchant initiated debit", sizeof(dest_string));
+      else if (data.transactiontype==OFX_REPEATPMT)
+	strncpy(dest_string, "REPEATPMT: Repeating payment/standing order", sizeof(dest_string));
+      else if (data.transactiontype==OFX_OTHER)
+	strncpy(dest_string, "OTHER: Other", sizeof(dest_string));
+      else
+	strncpy(dest_string, "Unknown transaction type", sizeof(dest_string));
+      cout<<"    Transaction type: "<<dest_string<<"\n";
     }
-    cout<<"    Transaction type: "<<dest_string<<"\n";
-  }
+
+  
   if(data.date_initiated_valid==true){
     strftime(dest_string,sizeof(dest_string),"%c %Z",localtime(&(data.date_initiated)));
     cout<<"    Date initiated: "<<dest_string<<"\n";
@@ -272,59 +152,56 @@ int ofx_proc_transaction_cb(struct OfxTransactionData data)
   }
   if(data.fi_id_correction_action_valid==true){
     cout<<"    Action to take on the corrected transaction: ";
-    switch(data.fi_id_correction_action){
-    case data.DELETE : cout<<"DELETE\n";
-      break;
-    case data.REPLACE : cout<<"REPLACE\n";
-      break;
-    default:
+    if (data.fi_id_correction_action==DELETE)
+      cout<<"DELETE\n";
+    else if (data.fi_id_correction_action==REPLACE)
+      cout<<"REPLACE\n";
+    else
       cout<<"ofx_proc_transaction(): This should not happen!\n";
     }
-  }
   if(data.invtransactiontype_valid==true){
     cout<<"    Investment transaction type: ";
-    switch(data.invtransactiontype){
-    case data.OFX_BUYDEBT: strncpy(dest_string, "BUYDEBT (Buy debt security)", sizeof(dest_string));
-      break;
-     case data.OFX_BUYMF: strncpy(dest_string, "BUYMF (Buy mutual fund)", sizeof(dest_string));
-      break;
-    case data.OFX_BUYOPT: strncpy(dest_string, "BUYOPT (Buy option)", sizeof(dest_string));
-      break;
-    case data.OFX_BUYOTHER: strncpy(dest_string, "BUYOTHER (Buy other security type)", sizeof(dest_string));
-      break;
-    case data.OFX_BUYSTOCK: strncpy(dest_string, "BUYSTOCK (Buy stock))", sizeof(dest_string));
-      break;
-    case data.OFX_CLOSUREOPT: strncpy(dest_string, "CLOSUREOPT (Close a position for an option)", sizeof(dest_string));
-      break;
-    case data.OFX_INCOME: strncpy(dest_string, "INCOME (Investment income is realized as cash into the investment account)", sizeof(dest_string));
-      break;
-    case data.OFX_INVEXPENSE: strncpy(dest_string, "INVEXPENSE (Misc investment expense that is associated with a specific security)", sizeof(dest_string));
-      break;
-    case data.OFX_JRNLFUND: strncpy(dest_string, "JRNLFUND (Journaling cash holdings between subaccounts within the same investment account)", sizeof(dest_string));
-      break;
-    case data.OFX_MARGININTEREST: strncpy(dest_string, "MARGININTEREST (Margin interest expense)", sizeof(dest_string));
-      break;
-    case data.OFX_REINVEST: strncpy(dest_string, "REINVEST (Reinvestment of income)", sizeof(dest_string));
-      break;
-    case data.OFX_RETOFCAP: strncpy(dest_string, "RETOFCAP (Return of capital)", sizeof(dest_string));
-      break;
-    case data.OFX_SELLDEBT: strncpy(dest_string, "SELLDEBT (Sell debt security.  Used when debt is sold, called, or reached maturity)", sizeof(dest_string));
-      break;
-    case data.OFX_SELLMF: strncpy(dest_string, "SELLMF (Sell mutual fund)", sizeof(dest_string));
-      break;
-    case data.OFX_SELLOPT: strncpy(dest_string, "SELLOPT (Sell option)", sizeof(dest_string));
-      break;
-    case data.OFX_SELLOTHER: strncpy(dest_string, "SELLOTHER (Sell other type of security)", sizeof(dest_string));
-      break;
-    case data.OFX_SELLSTOCK: strncpy(dest_string, "SELLSTOCK (Sell stock)", sizeof(dest_string));
-      break;
-    case data.OFX_SPLIT: strncpy(dest_string, "SPLIT (Stock or mutial fund split)", sizeof(dest_string));
-      break;
-    case data.OFX_TRANSFER: strncpy(dest_string, "TRANSFER (Transfer holdings in and out of the investment account)", sizeof(dest_string));
-      break;
-    default: strncpy(dest_string, "ERROR, this investment transaction type is unknown.  This is a bug in ofxdump", sizeof(dest_string));
-      break;
-    }
+    if (data.invtransactiontype==OFX_BUYDEBT)
+      strncpy(dest_string, "BUYDEBT (Buy debt security)", sizeof(dest_string));
+    else if (data.invtransactiontype==OFX_BUYMF)
+      strncpy(dest_string, "BUYMF (Buy mutual fund)", sizeof(dest_string));
+    else if (data.invtransactiontype==OFX_BUYOPT)
+      strncpy(dest_string, "BUYOPT (Buy option)", sizeof(dest_string));
+    else if (data.invtransactiontype==OFX_BUYOTHER)
+      strncpy(dest_string, "BUYOTHER (Buy other security type)", sizeof(dest_string));
+    else if (data.invtransactiontype==OFX_BUYSTOCK)
+      strncpy(dest_string, "BUYSTOCK (Buy stock))", sizeof(dest_string));
+    else if (data.invtransactiontype==OFX_CLOSUREOPT)
+      strncpy(dest_string, "CLOSUREOPT (Close a position for an option)", sizeof(dest_string));
+    else if (data.invtransactiontype==OFX_INCOME)
+      strncpy(dest_string, "INCOME (Investment income is realized as cash into the investment account)", sizeof(dest_string));
+    else if (data.invtransactiontype==OFX_INVEXPENSE)
+      strncpy(dest_string, "INVEXPENSE (Misc investment expense that is associated with a specific security)", sizeof(dest_string));
+    else if (data.invtransactiontype==OFX_JRNLFUND)
+      strncpy(dest_string, "JRNLFUND (Journaling cash holdings between subaccounts within the same investment account)", sizeof(dest_string));
+    else if (data.invtransactiontype==OFX_MARGININTEREST)
+      strncpy(dest_string, "MARGININTEREST (Margin interest expense)", sizeof(dest_string));
+    else if (data.invtransactiontype==OFX_REINVEST)
+      strncpy(dest_string, "REINVEST (Reinvestment of income)", sizeof(dest_string));
+    else if (data.invtransactiontype==OFX_RETOFCAP)
+      strncpy(dest_string, "RETOFCAP (Return of capital)", sizeof(dest_string));
+    else if (data.invtransactiontype==OFX_SELLDEBT)
+      strncpy(dest_string, "SELLDEBT (Sell debt security.  Used when debt is sold, called, or reached maturity)", sizeof(dest_string));
+    else if (data.invtransactiontype==OFX_SELLMF)
+      strncpy(dest_string, "SELLMF (Sell mutual fund)", sizeof(dest_string));
+    else if (data.invtransactiontype==OFX_SELLOPT)
+      strncpy(dest_string, "SELLOPT (Sell option)", sizeof(dest_string));
+    else if (data.invtransactiontype==OFX_SELLOTHER)
+      strncpy(dest_string, "SELLOTHER (Sell other type of security)", sizeof(dest_string));
+    else if (data.invtransactiontype==OFX_SELLSTOCK)
+      strncpy(dest_string, "SELLSTOCK (Sell stock)", sizeof(dest_string));
+    else if (data.invtransactiontype==OFX_SPLIT)
+      strncpy(dest_string, "SPLIT (Stock or mutial fund split)", sizeof(dest_string));
+    else if (data.invtransactiontype==OFX_TRANSFER)
+      strncpy(dest_string, "TRANSFER (Transfer holdings in and out of the investment account)", sizeof(dest_string));
+    else 
+      strncpy(dest_string, "ERROR, this investment transaction type is unknown.  This is a bug in ofxdump", sizeof(dest_string));
+    
     cout<<dest_string<<"\n";
   }
   if(data.unique_id_valid==true){
@@ -334,10 +211,7 @@ int ofx_proc_transaction_cb(struct OfxTransactionData data)
     cout<<"    Format of the Unique ID: "<<data.unique_id_type<<"\n";
   }
   if(data.security_data_valid==true){
-    cout<<"    Security data is available:\n    START security_data content----------\n";
-    ofx_proc_security_cb(*(data.security_data_ptr));
-    cout<<"    END security_data content----------\n";
-
+    ofx_proc_security_cb(*(data.security_data_ptr), NULL );
   }
 
   if(data.server_transaction_id_valid==true){
@@ -365,7 +239,7 @@ int ofx_proc_transaction_cb(struct OfxTransactionData data)
   return 0;
 }//end ofx_proc_transaction()
 
-int ofx_proc_statement_cb(struct OfxStatementData data)
+int ofx_proc_statement_cb(struct OfxStatementData data, void * statement_data)
 {
   char dest_string[255];
   cout<<"ofx_proc_statement():\n";
@@ -404,14 +278,14 @@ int ofx_proc_statement_cb(struct OfxStatementData data)
   return 0;
 }//end ofx_proc_statement()
 
-int ofx_proc_account_cb(struct OfxAccountData data)
+int ofx_proc_account_cb(struct OfxAccountData data, void * account_data)
 {
   cout<<"ofx_proc_account():\n";
   if(data.account_id_valid==true){
     cout<<"    Account ID: "<<data.account_id<<"\n";
     cout<<"    Account name: "<<data.account_name<<"\n";
   }
-   if(data.account_type_valid==true){
+  if(data.account_type_valid==true){
     cout<<"    Account type: ";
     switch(data.account_type){
     case OfxAccountData::OFX_CHECKING : cout<<"CHECKING\n";
@@ -438,3 +312,103 @@ int ofx_proc_account_cb(struct OfxAccountData data)
   cout<<"\n";
   return 0;
 }//end ofx_proc_account()
+
+
+
+int ofx_proc_status_cb(struct OfxStatusData data, void * status_data)
+{
+  cout<<"ofx_proc_status():\n";
+  if(data.ofx_element_name_valid==true){
+    cout<<"    Ofx entity this status is relevent to: "<< data.ofx_element_name<<" \n";
+  }
+  if(data.severity_valid==true){
+    cout<<"    Severity: ";
+    switch(data.severity){
+    case OfxStatusData::INFO : cout<<"INFO\n";
+      break;
+    case OfxStatusData::WARN : cout<<"WARN\n";
+      break;
+    case OfxStatusData::ERROR : cout<<"ERROR\n";
+      break;
+    default: cout<<"WRITEME: Unknown status severity!\n";
+    }
+  }
+  if(data.code_valid==true){
+    cout<<"    Code: "<<data.code<<", name: "<<data.name<<"\n    Description: "<<data.description<<"\n";
+  }
+  if(data.server_message_valid==true){
+    cout<<"    Server Message: "<<data.server_message<<"\n";
+  }
+  cout<<"\n";
+  return 0;
+}
+
+
+int main (int argc, char *argv[])
+{
+  /** Tell ofxdump what you want it to send to stderr.  See messages.cpp for more details */
+  extern int ofx_PARSER_msg;
+  extern int ofx_DEBUG_msg;
+  extern int ofx_WARNING_msg;
+  extern int ofx_ERROR_msg;
+  extern int ofx_INFO_msg;
+  extern int ofx_STATUS_msg;
+
+  gengetopt_args_info args_info;
+
+  /* let's call our cmdline parser */
+  if (cmdline_parser (argc, argv, &args_info) != 0)
+    exit(1) ;
+
+  //  if (args_info.msg_parser_given)
+  //    cout << "The msg_parser option was given!" << endl;
+
+  //  cout << "The flag is " << ( args_info.msg_parser_flag ? "on" : "off" ) <<
+  //    "." << endl ;
+  args_info.msg_parser_flag ? ofx_PARSER_msg = true : ofx_PARSER_msg = false;
+  args_info.msg_debug_flag ? ofx_DEBUG_msg = true : ofx_DEBUG_msg = false;
+  args_info.msg_warning_flag ? ofx_WARNING_msg = true : ofx_WARNING_msg = false;
+  args_info.msg_error_flag ? ofx_ERROR_msg = true : ofx_ERROR_msg = false;
+  args_info.msg_info_flag ? ofx_INFO_msg = true : ofx_INFO_msg = false;
+  args_info.msg_status_flag ? ofx_STATUS_msg = true : ofx_STATUS_msg;
+
+  if(args_info.list_import_formats_given)
+    {
+      cout <<"The supported file formats for the 'input-file-format' argument are:"<<endl;
+      for(int i=0; LibofxImportFormatList[i].format!=LAST; i++)
+	{
+	  cout <<"     "<<LibofxImportFormatList[i].description<<endl;
+	}
+    }
+
+  LibofxContextPtr libofx_context = libofx_get_new_context();
+
+  //char **inputs ; /* unamed options */
+  //unsigned inputs_num ; /* unamed options number */
+  if (args_info.inputs_num  > 0)
+    {
+
+      ofx_set_statement_cb(libofx_context, ofx_proc_statement_cb, 0);
+      ofx_set_account_cb(libofx_context, ofx_proc_account_cb, 0);
+      ofx_set_transaction_cb(libofx_context, ofx_proc_transaction_cb, 0);
+      ofx_set_security_cb(libofx_context, ofx_proc_security_cb, 0);
+      ofx_set_status_cb(libofx_context, ofx_proc_status_cb, 0);
+
+      enum LibofxFileFormat file_format = libofx_get_file_format_from_str(LibofxImportFormatList, args_info.import_format_arg);
+      /** @todo currently, only the first file is processed as the library can't deal with more right now.*/
+      if(args_info.inputs_num  > 1)
+        {
+          cout << "Sorry, currently, only the first file is processed as the library can't deal with more right now.  The followinf files were ignored:"<<endl;
+          for ( unsigned i = 1 ; i < args_info.inputs_num ; ++i )
+            {
+              cout << "file: " << args_info.inputs[i] << endl ;
+	      }
+        }
+      libofx_proc_file(libofx_context, args_info.inputs[0], file_format);
+    }
+  else
+    {
+
+    }
+  return 0;
+}
